import { Nullable } from '../utils';
export interface IMapAccess {
    nextKey<T>(seed?: Deserialize<T>): IteratorResult<T>;
    nextValue<T>(seed?: Deserialize<T>): IteratorResult<T>;
    nextEntry<K, V>(kseed?: Deserialize<K>, vseed?: Deserialize<V>): IteratorResult<[K, V]>;
    sizeHint?(): Nullable<number>;
    entries<T>(seed?: Deserialize<T>): Iterator<T>;
    values<T>(seed?: Deserialize<T>): Iterator<T>;
    entries<K, V>(kseed: Deserialize<K>, vseed: Deserialize<V>): Iterator<[K, V]>;
    [Symbol.iterator]<K, V>(): Iterator<[K, V]>;
}
export declare abstract class MapAccess {
    abstract nextKey<T>(seed?: Deserialize<T>): IteratorResult<T>;
    abstract nextValue<T>(seed?: Deserialize<T>): IteratorResult<T>;
    nextEntry<K, V>(kseed?: Deserialize<K>, vseed?: Deserialize<V>): IteratorResult<[K, V]>;
    private generate;
    keys<T>(seed?: Deserialize<T>): Iterator<T>;
    values<T>(seed?: Deserialize<T>): Iterator<T>;
    entries<K, V>(kseed?: Deserialize<K>, vseed?: Deserialize<V>): Iterator<[K, V]>;
    [Symbol.iterator]<K, V>(): Iterator<K, V>;
}
export interface IIterableAccess {
    nextElement<T>(seed: Deserialize<T>): IteratorResult<T>;
    sizeHint?(): Nullable<number>;
    [Symbol.iterator]<T>(): Iterator<T>;
}
export declare abstract class IterableAccess implements IIterableAccess {
    abstract nextElement<T>(seed: Deserialize<T>): IteratorResult<T>;
    [Symbol.iterator]<T>(): Iterator<T>;
}
export declare function isVisitor<T>(visitor: any): visitor is IVisitor<T>;
export interface IVisitor<T> {
    visitBoolean(value: boolean): T;
    visitNumber(value: number): T;
    visitBigInt(value: bigint): T;
    visitString(value: string): T;
    visitSymbol(value: symbol): T;
    visitNull(): T;
    visitObject(access: IMapAccess): T;
    visitIterable(access: IIterableAccess): T;
}
export interface IDeserializer {
    deserializeAny<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeBoolean<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeNumber<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeBigInt<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeString<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeSymbol<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeNull<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeObject<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeIterable<T>(visitor: Partial<IVisitor<T>>): T;
    deserializeFunction<T>(visitor: Partial<IVisitor<T>>): T;
}
export interface Deserialize<T> {
    (deserializer: IDeserializer): T;
}
