import { ISerializeIterable, ISerializeObject, ISerializer, SerializeIterable, SerializeObject } from './interface'

class IdentityMap<T> extends SerializeObject<T> {
  private value: Record<any, any> = {}
  private currentKey?: string

  serializeKey(key: string): void {
    this.currentKey = key
  }

  serializeValue<U>(value: U): void {
    this.value[this.currentKey!] = value
  }

  end(): T {
    return this.value
  }
}

class IdentityIterable<T> extends SerializeIterable<T> {
  private value: any[] = []

  serializeElement<U>(value: U): void {
    this.value.push(value)
  }

  end(): T {
    return this.value as T
  }
}

export class IdentitySerializer<T> implements ISerializer<T> {
  serializeAny?(value: any): T {
    return value as T
  }

  serializeBoolean(value: boolean): T {
    return value as T
  }

  serializeNumber(value: number): T {
    return value as T
  }

  serializeBigInt(value: bigint): T {
    return value as T
  }

  serializeString(value: string): T {
    return value as T
  }

  serializeSymbol(value: symbol): T {
    return value as T
  }

  serializeNull(): T {
    return null as T
  }

  serializeIterable(_len?: number): ISerializeIterable<T> {
    return new IdentityIterable()
  }

  serializeObject(_len?: number): ISerializeObject<T> {
    return new IdentityMap()
  }

  serializeClass(_name: string, _len?: number): ISerializeObject<T> {
    return new IdentityMap()
  }
}

